/* Header for module ToolPalette					 */

#ifndef ToolPalette_H
#define ToolPalette_H

/* ------------- Mac System IMPORTS ------------------*/


#include <Carbon/Carbon.h>

/* Palette Manager */  

#define maxCells        100

typedef enum {
	eMDown=0, eMUp, eDraw,eDummy=257
} tAction;

typedef enum {
	simplePalette, subPalette, popUpPalette
} tPaletteType;

typedef enum {
	simpleItem, complexItem, subPaletteItem, popUpPaletteItem
} tToolItemType;

typedef enum {
	none, invert_, grayFill, invertFrame
} tHiliteStyle;

typedef enum {
	noFrame, wireFrame, shadowFrame
} tFrameStyle;

typedef enum {
	NONE, SICN, ICON, PICT, PAT, COLOURFILL
} tItemRes;

/* IF YOU MAKE CHANGES TO THE FOLLOWING STRUCTURES PLEASE REMEMBER TO CHANGE THE CORRESPONDING */
/*  Resource Definition																		   */ 

struct tCellDef{
	tToolItemType 	itemType;				/* simple, complex, subPalette, popUpPalette */
	tItemRes 		itemResType;			/* resouce type - used to do drawing of item. ICON,SICN,PICT,PAT */
	Boolean 		copyBackFlag;			/* used if this is a PopUp Palette			 */
	short 			paletteResNum;			/* Resource Id Number for this Palette       */
	struct tPalette *subPalettePtr;   		/* Pointer to Sub Palette(ie PopUp ) */
	Rect 			hitBox;   				/* Rectangle where hit testing is done for this tool*/
	Rect 			drawRect;   			/* Rectangle to draw tool in , muse =< hitBox */
	Boolean 		enabled;   				/* Will respond to mouse down/up if true */
	tFrameStyle 	frame;					/* true IF you want to frame each tool Default: true */
	short			returnVal;   			/* Integer value returned when this cell is hit */
	short 			cursorID;				/* ID of Cursor that should be in effect once this item is Hit */
	short 			icnNum;					/* This is used in drawing this item, the name is a little */
											/* miss leading, as it is all so used for ICON, SICN and COlOUR */
											/* This identifies what ICON, SICN or Colour should be drawn   */
	BitMap 			icnBitMap;  			/* IF item is draw with a BitMap- store it here */
	short 			patListID;				/* IF item is a pattern then this is the Pattern list ID */
	Pattern 		pattern;				/* Store the actual  pattern */
	short 			itemActionResID;		/* Resource number of the Action Routine for this Cell */
	
	/* Action Routine Pointer */
	
	pascal Boolean 		(*itemActionProc)(tAction, struct tPalette *, Point, Point, short,
							          short, short *, short *, short *, short *);
									  
	Picture 		**picHandle;			/* Pictuer Handle - only valid if itemType is "PICT" */
} ;

typedef struct tCellDef tCellDef;

struct tPalette{
	short 			valid;			/* This is used to verify that data from resource is valid - auto initialized in resource*/
	tPaletteType	paletteType;    /* filled in by program */
	unsigned short 	paletteID; 		/* if palette created from resource then this will be the res number, else its user specified - filled in by program**/
	WindowPtr		paletteWindow;  /* The window this palette is in- filled in by program */
	short 			lastHit;   		/* index of last hit tool in Palette - filled in by program*/
	tHiliteStyle 	hiliteStyle;    /* filled in by resource */
	Rect 			paletteRect;	/*This rectangle encloses the complete tool pallette. - filled in by resource*/		
	short 			rows, coloums;  /* Number of rows and coloums making up Palette - filled in by resource*/
	tItemRes		mItemResType;   /* resource type used to do drawing of item  - filled in by resource */
									/* This is the master ItemResTYpe which is used */
									/* for simple tool palette items.Complex and popUp */
									/* have one for each item in the tool palette */

	tFrameStyle 	frameStyle; 		/* noFrame, wireFrame or shadowFrame - filled in by resource */
	Boolean 		hide;   			/* true IF Palette is hidden(will not be drawn) - filled in by resource */
	struct tPalette *parentPalette;		/* Pointer to Parent Palette if this is a sub aplette - filled in by program*/
	short 			pItemActionResID;	/* Palette Level Action Routines resource Number - filled in by resource */
	
	/* Palette Level action routine. This will be used for all items in the Palette unless  */
	/* the item has its own Action Routine. If it does,  this routine will still be called   */
    /* if the item one returns true 														*/ 
	
	 pascal Boolean 		(*pItemActionProc)(tAction, struct tPalette *, Point, Point,
							   			short, short, short *, short *, short *,
							   			short *);  /* filled in by software */
	BitMap 			palBitMap;			/* filled in by software */
	short 			numOfCells;   		/* number of cells in use may be less then rows*col - autofilled by resource*/
	tCellDef 		cells[maxCells];   /* individual tool  parameters */
} ;

typedef struct tPalette tPalette;

extern Str255 paletteErrString;
extern short  paletteErrNum;
#ifdef __cplusplus
extern "C" {
#endif



							/* ROUTINES */

/************************************************************/
/* This ROUTINE MUST be Called before you can use this Package  */
/************************************************************/

extern pascal void ToolPalette_Init(short palErrAlertID);

/************************************************************/
/* Read in a palette definition from a resource. 			*/
/* Passed in -     a Pallete Definition resource ID     	*/
/* Returns 	 -      pointer to a Palette Structure			*/
/* 		     - 	  	an error number (0 if everything worked)*/
/************************************************************/

extern pascal  tPalette *CCTNewPalette(unsigned short paletteID, short *paletteErrNum);


/************************************************************/
/* Draws the the Palette that is pointed to by palettePtr.	*/
/************************************************************/

extern pascal  void PaletteDraw(tPalette *palettePtr, Point topCorner, short *paletteErrNum);


/************************************************************/
/* Draw all the palettes that have been loaded 				*/
/************************************************************/

extern pascal  void DrawAllPalettes(Rect updateRect);

/************************************************************/
/* Determine which palette and item was hit if any			*/
/* Passed in 	- 	pointer to a window						*/
/* 				-   mouse Location							*/
/************************************************************/

extern pascal  short PaletteHit(WindowPtr window, Point where, short *newCursorID,
						tPalette **palettePtr, short *palID,
						short *paletteErrNum);


/************************************************************/						
/*	Get rid of a palette and release its space				*/
/************************************************************/

extern pascal  void PaletteDispose(tPalette *palettePtr);

/************************************************************/
/* Move the Palette pointed to by  palettePtr to "postion " */
/************************************************************/

extern pascal  void PaletteMove(tPalette *palettePtr, Point position, short *paletteErrNum);

/************************************************************/
/* Changes the hilited cell in the tool palette to be the   */
/* cell who's return value is "cellValue"                   */
/* Returns the new Cursor ID of "cellValue"					*/
/************************************************************/

extern pascal  short ChangePaletteHilitedCell(tPalette *palettePtr, short cellValue);

/************************************************************/
/* Change the status of cell from selectabel to unselectable*/
/* or vise versa 											*/
/* Passed in - onOff - true-> selectable					*/
/*				     - false-> unselectable					*/
/************************************************************/

extern pascal void PaletteCellSelectable(tPalette *palettePtr, short cellValue, Boolean onOff);


/************************************************************/
/* Find the sub Palette pointer for Palette who resource    */
/* ID is "palID" and who's parent palette is "parentPalette */
/************************************************************/

extern pascal tPalette *FindSubPalettePointer(short palID, tPalette* parentPalette);


/************************************************************/
/* Print out in Debug window the palette Header info        */
/************************************************************/

extern pascal  void DumpPalette(tPalette *palettePtr);


/************************************************************/
/* Passed a pointer to a toolPtr and a mouse down location  */
/*  this routing will return the ordinal number of the tool */
/* that was hit.											*/
/************************************************************/


extern pascal  void PalHit(Point where, Point topCorner, short *newCursorID,
				   tPalette **palettePtr, short *palID, short *item,
				   short *cellHit, short *paletteErrNum);
				   



extern pascal  void CellDraw(tPalette *palettePtr, Point topCorner, short item,short paletteErrNum);

/************************************************************/
/* This routine is used to set the Cursor ID number for each*/ 
/* tool in the palette 										*/
/* This will be used to set  the cursor to the appropriate	*/
/*										shape when  mouse is */
/* in content routine but not in a tool palette     		 */
/************************************************************/

extern pascal  void SetPaletteCursors(tPalette *palettePtr, short startIndex, short endIndex, short cursorResID);

/************************************************************/
/* Read in a Small Icon definition from a resource 			*/
/************************************************************/

extern pascal  Boolean GetSICN(BitMap *theBitMap, short theResID);

/************************************************************/ 
/* Draw the small ICON on the screen 						*/
/************************************************************/

extern pascal  void DrawSICN(BitMap srcBits, Rect dstRect, short mode);
 

#ifdef __cplusplus
}
#endif

#endif 

